﻿using UnityEngine;

[RequireComponent(typeof(ActorMovement))]
[RequireComponent(typeof(PlayerState))]
[RequireComponent(typeof(Rigidbody2D))]
public class PlayerInput : MonoBehaviour
{
    [Header("Keyboard Inputs")]
    [SerializeField]
    private string keyMoveXAxis;
    [SerializeField]
    private string keyMoveYAxis;
    [SerializeField]
    private string keyPickupButton;

    [Header("Joypad Inputs")]
    [SerializeField]
    private string joyMoveXAxis;
    [SerializeField]
    private string joyMoveYAxis;
    [SerializeField]
    private string joyPickupButton;

    // Variables not shown in Inspector.
    private Controller activeControl;

    private ActorMovement movement;
    private PlayerState state;

    private void Start()
    {
        activeControl = Controller.KEYBOARD;

        movement = GetComponent<ActorMovement>();
        state = GetComponent<PlayerState>();
    }

    private void Update()
    {
        float moveX, moveY;
        activeControl = DominantControl();
        switch (activeControl)
        {
            case Controller.KEYBOARD:
                moveX = Input.GetAxis(keyMoveXAxis);
                moveY = Input.GetAxis(keyMoveYAxis);
                movement.MoveInput(moveX, moveY);

                if (Input.GetButtonDown(keyPickupButton))
                    state.PickupInput();
                break;
            case Controller.JOYPAD:
                moveX = Input.GetAxis(joyMoveXAxis);
                moveY = Input.GetAxis(joyMoveYAxis);
                movement.MoveInput(moveX, moveY);

                if (Input.GetButtonDown(joyPickupButton))
                    state.PickupInput();
                break;
        }
    }

    // Determine which controller is most dominant.
    private Controller DominantControl()
    {
        if (activeControl == Controller.JOYPAD
            && (Mathf.Abs(Input.GetAxis(keyMoveXAxis)) > 0.1f
            || Mathf.Abs(Input.GetAxis(keyMoveYAxis)) > 0.1f))
            return Controller.KEYBOARD;
        else if (activeControl == Controller.KEYBOARD
            && (Mathf.Abs(Input.GetAxis(joyMoveXAxis)) > 0.1f
            || Mathf.Abs(Input.GetAxis(joyMoveYAxis)) > 0.1f))
            return Controller.JOYPAD;

        // If other controller not dominant, keep current controller.
        return activeControl;
    }

    private enum Controller
    {
        JOYPAD,
        KEYBOARD
    }
}