﻿using UnityEngine;

public class PlayerState : MonoBehaviour
{
    public static PlayerState state;

    private int framesNotInTrigger = 5;

    private bool inTrigger = false;
    private bool inTemple = false;
    private bool canPickup = false;

    [SerializeField]
    private Canvas indicator;

    private GainItem item;
    private CraftableItem requirement;
    private BlockedArea area;

    private void Start()
    {
        state = this;
    }

    public void SetInTrigger(GainItem item)
    {
        inTrigger = true;
        framesNotInTrigger = 0;
        this.item = item;
        //PickupInput();
    }

    public void SetInTrigger(CraftableItem requirement, BlockedArea area)
    {
        inTrigger = true;
        framesNotInTrigger = 0;
        this.requirement = requirement;
        this.area = area;
        //PickupInput();
    }

    public void SetCanUseTemple()
    {
        inTemple = true;
        framesNotInTrigger = 0;
    }

    public void PickupInput()
    {
        if (canPickup && inTrigger)
        {
            if (item != null)
                item.PickupItem();
            else
                TryPlacing();
        }
        else if (canPickup && inTemple)
            UseTemple.temple.Interact();
    }

    private void LateUpdate()
    {
        if (!inTrigger && !inTemple)
            ++framesNotInTrigger;

        canPickup = (framesNotInTrigger < 5);
        inTrigger = false;
        inTemple = false;

        if (!canPickup)
            item = null;

        indicator.gameObject.SetActive(canPickup);
    }

    private void TryPlacing()
    {
        if (requirement.IsUnlocked())
            area.Unlock();
        else
            area.Fail();
    }
}