﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;

public class CraftingController : MonoBehaviour
{
    [SerializeField]
    private Slot slot1, slot2;

    private bool canExit = false;

    [SerializeField]
    private Recipe[] recipes;
    [SerializeField]
    private CraftableItem[] defaultItems;

    [SerializeField]
    private CraftableItem[] allItems;
    [SerializeField]
    private ItemSlot[] allSlots;

    private bool lastItemAlreadyUnlocked;
    private bool unlockedLife = false;
    [SerializeField]
    private CraftableItem life;

    [SerializeField]
    private CraftDialogue dialogue;

    [SerializeField]
    private Image exitBtn;
    [SerializeField]
    private Sprite exitImg;

    [SerializeField]
    private Canvas cv;

    private bool hasHadAnEnable = false;

    private void Start()
    {
        for (int i = 0; i < allItems.Length; ++i)
            allItems[i].item = allSlots[i];

        for (int i = 0; i < defaultItems.Length; ++i)
            defaultItems[i].Unlock();
    }

    private void OnEnable()
    {
        if(hasHadAnEnable)
            GameController.gc.SetState(GameState.CRAFTING);
        slot1.Clear();
        slot2.Clear();
        hasHadAnEnable = true;
    }

    public CraftableItem CheckRecipe(CraftableItem item1, CraftableItem item2)
    {
        for (int i = 0; i < recipes.Length; ++i)
        {
            if (recipes[i].CheckRecipe(item1, item2))
            {
                lastItemAlreadyUnlocked = recipes[i].result.Unlock();

                return recipes[i].result;
            }
        }

        return null;
    }

    public void SetItem(CraftableItem item)
    {
        if (slot1.GetItem() == null)
            slot1.SetItem(item);
        else if(slot2.GetItem() == null)
        {
            StartCoroutine(SetSecondItem(item));
        }
    }

    public IEnumerator SetSecondItem(CraftableItem item)
    {
        slot2.SetItem(item);
        CraftableItem res = CheckRecipe(slot1.GetItem(), slot2.GetItem());
        if (res == null)
        {
            yield return new WaitForSeconds(0.5f);
            slot1.Clear();
            slot2.Clear();
        }
        else
        {
            res.Unlock();
            dialogue.gameObject.SetActive(true);
            dialogue.SetItem(res);
            GameController.gc.SetState(GameState.CRAFT_MSG);

            yield return StartCoroutine(WaitForEndOfDialogue());
            slot1.Clear();
            slot2.Clear();
        }

        if (!unlockedLife && res == life)
        {
            unlockedLife = true;

            exitBtn.sprite = exitImg;
        }
    }

    private IEnumerator WaitForEndOfDialogue()
    {
        while (GameController.gc.GetState() != GameState.CRAFTING)
            yield return new WaitForEndOfFrame();
    }

    public void Exit()
    {
        if (unlockedLife)
        {
            GameController.gc.SetState(GameState.OVERWORLD);
            cv.gameObject.SetActive(false);
        }
    }
}